@@ -1,22 +1,9 @@
-/**************************************************************************
- * Copyright (C) 2009 Ben Cooksley <bcooksley@kde.org>                    *
- * Copyright (C) 2007 Will Stephenson <wstephenson@kde.org>               *
- *                                                                        *
- * This program is free software; you can redistribute it and/or          *
- * modify it under the terms of the GNU General Public License            *
- * as published by the Free Software Foundation; either version 2         *
- * of the License, or (at your option) any later version.                 *
- *                                                                        *
- * This program is distributed in the hope that it will be useful,        *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- * GNU General Public License for more details.                           *
- *                                                                        *
- * You should have received a copy of the GNU General Public License      *
- * along with this program; if not, write to the Free Software            *
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA          *
- * 02110-1301, USA.                                                       *
- ***************************************************************************/
+/*
+ * SPDX-FileCopyrightText: 2009 Ben Cooksley <bcooksley@kde.org>
+ * SPDX-FileCopyrightText: 2007 Will Stephenson <wstephenson@kde.org>
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include "MenuProxyModel.h"
 
@@ -83,17 +70,27 @@ bool MenuProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_
         if (mItem->menu() && mItem->children().isEmpty()) {
             return false;
         }
-        return KCategorizedSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
+        if (mItem->service()->library() == QLatin1String("kcm_landingpage")) {
+            return false;
+        } else {
+            return KCategorizedSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
+        }
     }
 
     QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
     MenuItem *mItem = index.data(Qt::UserRole).value<MenuItem *>();
+
+    if (mItem->service()->library() == QLatin1String("kcm_landingpage")) {
+        return false;
+    }
+
     // accept only systemsettings categories that have children
     if (mItem->children().isEmpty() && mItem->service()->serviceTypes().contains(QLatin1String("SystemSettingsCategory"))) {
         return false;
     } else {
         return true; // Items matching the regexp are disabled, not hidden
     }
+
 }
 
 void MenuProxyModel::setFilterHighlightsEntries(bool highlight)
@@ -125,10 +122,10 @@ void MenuProxyModel::setFilterRegExp(const QString &pattern)
     if (pattern == filterRegExp()) {
         return;
     }
-    emit layoutAboutToBeChanged();
+    Q_EMIT layoutAboutToBeChanged();
     KCategorizedSortFilterProxyModel::setFilterRegExp(pattern);
-    emit layoutChanged();
-    emit filterRegExpChanged();
+    Q_EMIT layoutChanged();
+    Q_EMIT filterRegExpChanged();
 }
 
 QString MenuProxyModel::filterRegExp() const
@@ -138,7 +135,7 @@ QString MenuProxyModel::filterRegExp() const
 
 void MenuProxyModel::setFilterRegExp(const QRegExp &regExp)
 {
-    emit layoutAboutToBeChanged();
+    Q_EMIT layoutAboutToBeChanged();
     KCategorizedSortFilterProxyModel::setFilterRegExp(regExp);
-    emit layoutChanged();
+    Q_EMIT layoutChanged();
 }
