@@ -1,21 +1,8 @@
-/**************************************************************************
- * Copyright (C) 2009 by Ben Cooksley <bcooksley@kde.org>                 *
- *                                                                        *
- * This program is free software; you can redistribute it and/or          *
- * modify it under the terms of the GNU General Public License            *
- * as published by the Free Software Foundation; either version 2         *
- * of the License, or (at your option) any later version.                 *
- *                                                                        *
- * This program is distributed in the hope that it will be useful,        *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
- * GNU General Public License for more details.                           *
- *                                                                        *
- * You should have received a copy of the GNU General Public License      *
- * along with this program; if not, write to the Free Software            *
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA          *
- * 02110-1301, USA.                                                       *
- ***************************************************************************/
+/*
+ * SPDX-FileCopyrightText: 2009 Ben Cooksley <bcooksley@kde.org>
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #include "SidebarMode.h"
 
@@ -32,6 +19,7 @@
 #include <KAboutData>
 #include <KActionCollection>
 #include <KCModuleInfo>
+#include <KCMUtils/KCModuleLoader>
 #include <KConfigGroup>
 #include <KDeclarative/KDeclarative>
 #include <KDescendantsProxyModel>
@@ -98,7 +86,7 @@ QString SubcategoryModel::title() const
         return QString();
     }
 
-    return mi->item().moduleName();
+    return mi->name();
 }
 
 QIcon SubcategoryModel::icon() const
@@ -115,9 +103,9 @@ void SubcategoryModel::setParentIndex(const QModelIndex &activeModule)
 {
     selectionModel()->select(activeModule, QItemSelectionModel::ClearAndSelect);
     m_activeModuleIndex = QPersistentModelIndex(activeModule);
-    emit titleChanged();
-    emit iconChanged();
-    emit categoryOwnedByKCMChanged();
+    Q_EMIT titleChanged();
+    Q_EMIT iconChanged();
+    Q_EMIT categoryOwnedByKCMChanged();
 }
 
 void SubcategoryModel::loadParentCategoryModule()
@@ -288,7 +276,7 @@ public:
             return;
         }
         m_actionMenuVisible = actionMenuVisible;
-        emit sidebarMode->actionMenuVisibleChanged();
+        Q_EMIT sidebarMode->actionMenuVisibleChanged();
     }
     bool m_introPageVisible = true;
     bool m_defaultsIndicatorsVisible = false;
@@ -445,7 +433,9 @@ void SidebarMode::requestMostUsedToolTip(int index, const QRectF &rect)
 
 void SidebarMode::hideToolTip()
 {
-    d->toolTipManager->hideToolTip();
+    if (d->toolTipManager) {
+        d->toolTipManager->hideToolTip();
+    }
 }
 
 void SidebarMode::hideMostUsedToolTip()
@@ -499,7 +489,7 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
 
     if (homeItem()) {
         d->m_introPageVisible = activeModule == d->categorizedModel->mapFromSource(d->model->indexForItem(homeItem()));
-        emit introPageVisibleChanged();
+        Q_EMIT introPageVisibleChanged();
     } else {
         setIntroPageVisible(false);
     }
@@ -512,28 +502,24 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
         d->activeCategoryIndex = activeModule;
         d->activeCategoryRow = newCategoryRow;
 
-        if (mi->item().library().isEmpty()) {
-            d->activeSubCategoryRow = 0;
-        } else {
-            d->activeSubCategoryRow = -1;
-        }
+        d->activeSubCategoryRow = 0;
 
         d->subCategoryModel->setParentIndex(activeModule);
 
         if (d->activeSearchRow > -1) {
             d->activeSearchRow = -1;
-            emit activeSearchRowChanged();
+            Q_EMIT activeSearchRowChanged();
         }
-        emit activeCategoryRowChanged();
-        emit activeSubCategoryRowChanged();
+        Q_EMIT activeCategoryRowChanged();
+        Q_EMIT activeSubCategoryRowChanged();
 
     } else if (activeModule.model() == d->subCategoryModel) {
         if (d->activeSearchRow > -1) {
             d->activeSearchRow = -1;
-            emit activeSearchRowChanged();
+            Q_EMIT activeSearchRowChanged();
         }
         d->activeSubCategoryRow = activeModule.row();
-        emit activeSubCategoryRowChanged();
+        Q_EMIT activeSubCategoryRowChanged();
 
     } else if (activeModule.model() == d->searchModel) {
         QModelIndex originalIndex = d->categorizedModel->mapFromSource(d->flatModel->mapToSource(d->searchModel->mapToSource(activeModule)));
@@ -551,17 +537,17 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
             }
 
             d->subCategoryModel->setParentIndex(originalIndex.parent().isValid() ? originalIndex.parent() : originalIndex);
-            emit activeCategoryRowChanged();
-            emit activeSubCategoryRowChanged();
+            Q_EMIT activeCategoryRowChanged();
+            Q_EMIT activeSubCategoryRowChanged();
         }
 
         d->activeSearchRow = activeModule.row();
-        emit activeSearchRowChanged();
+        Q_EMIT activeSearchRowChanged();
 
     } else {
         if (d->activeSearchRow > -1) {
             d->activeSearchRow = -1;
-            emit activeSearchRowChanged();
+            Q_EMIT activeSearchRowChanged();
         }
 
         QModelIndex flatIndex;
@@ -594,8 +580,8 @@ void SidebarMode::loadModule(const QModelIndex &activeModule, const QStringList
                     d->activeCategoryRow = idx.row();
                     d->activeSubCategoryRow = -1;
                 }
-                emit activeCategoryRowChanged();
-                emit activeSubCategoryRowChanged();
+                Q_EMIT activeCategoryRowChanged();
+                Q_EMIT activeSubCategoryRowChanged();
             }
         }
     }
@@ -615,6 +601,10 @@ void SidebarMode::moduleLoaded()
 
 void SidebarMode::updateDefaults()
 {
+    // When the landing page is loaded, we don't have activeCategoryRow
+    if (d->activeCategoryRow < 0) {
+        return;
+    }
     QModelIndex categoryIdx = d->categorizedModel->index(d->activeCategoryRow, 0);
     auto item = categoryIdx.data(Qt::UserRole).value<MenuItem *>();
     Q_ASSERT(item);
@@ -623,14 +613,25 @@ void SidebarMode::updateDefaults()
         auto subCateogryIdx = d->subCategoryModel->index(d->activeSubCategoryRow, 0);
         item = subCateogryIdx.data(Qt::UserRole).value<MenuItem *>();
     }
-    item->updateDefaultIndicator();
 
+    auto *moduleData = KCModuleLoader::loadModuleData(item->item());
+    if (moduleData) {
+        connect(moduleData, &KCModuleData::loaded, this, [this, item, moduleData, categoryIdx]() {
+            item->setDefaultIndicator(!moduleData->isDefaults());
+            updateCategoryModel(categoryIdx);
+            moduleData->deleteLater();
+        });
+    }
+}
+
+void SidebarMode::updateCategoryModel(const QModelIndex &categoryIdx)
+{
     auto sourceIdx = d->categorizedModel->mapToSource(categoryIdx);
-    emit d->model->dataChanged(sourceIdx, sourceIdx);
+    Q_EMIT d->model->dataChanged(sourceIdx, sourceIdx);
 
     auto subCateogryIdx = d->subCategoryModel->index(d->activeSubCategoryRow, 0);
     auto subCategorySourceIdx = d->categorizedModel->mapToSource(d->subCategoryModel->mapToSource(subCateogryIdx));
-    emit d->model->dataChanged(subCategorySourceIdx, subCategorySourceIdx);
+    Q_EMIT d->model->dataChanged(subCategorySourceIdx, subCategorySourceIdx);
 }
 
 int SidebarMode::activeSearchRow() const
@@ -654,15 +655,25 @@ void SidebarMode::setIntroPageVisible(const bool &introPageVisible)
         d->placeHolderWidget->hide();
         d->moduleView->show();
         if (introPageVisible) {
-            loadModule(d->categorizedModel->mapFromSource(d->model->indexForItem(homeItem())));
+            QModelIndex index = d->categorizedModel->mapFromSource(d->model->indexForItem(homeItem()));
+            if (index.isValid()) {
+                loadModule(index);
+            } else {
+                d->moduleView->closeModules();
+                d->moduleView->loadModule( d->model->indexForItem(homeItem()), QStringList() );
+                d->activeCategoryRow = -1;
+                d->activeSubCategoryRow = -1;
+                Q_EMIT activeCategoryRowChanged();
+                Q_EMIT activeSubCategoryRowChanged();
+            }
         }
     } else {
         if (introPageVisible) {
             d->subCategoryModel->setParentIndex(QModelIndex());
             d->activeCategoryRow = -1;
-            emit activeCategoryRowChanged();
+            Q_EMIT activeCategoryRowChanged();
             d->activeSubCategoryRow = -1;
-            emit activeSubCategoryRowChanged();
+            Q_EMIT activeSubCategoryRowChanged();
             d->placeHolderWidget->show();
             d->moduleView->hide();
         } else {
@@ -672,7 +683,7 @@ void SidebarMode::setIntroPageVisible(const bool &introPageVisible)
     }
 
     d->m_introPageVisible = introPageVisible;
-    emit introPageVisibleChanged();
+    Q_EMIT introPageVisibleChanged();
 }
 
 void SidebarMode::setHeaderHeight(qreal height)
@@ -682,7 +693,7 @@ void SidebarMode::setHeaderHeight(qreal height)
     }
 
     d->moduleView->setHeaderHeight(height);
-    emit headerHeightChanged();
+    Q_EMIT headerHeightChanged();
 }
 
 qreal SidebarMode::headerHeight() const
@@ -702,28 +713,35 @@ void SidebarMode::toggleDefaultsIndicatorsVisibility()
             if (item->menu()) {
                 continue;
             }
-            const bool showIndicator = item->showDefaultIndicator();
-            item->updateDefaultIndicator();
-            if (showIndicator == item->showDefaultIndicator()) {
-                continue;
-            }
-            auto itemIdx = d->model->indexForItem(item);
-            emit d->model->dataChanged(itemIdx, itemIdx);
-            MenuItem *parent = item->parent();
-            while (parent) {
-                auto parentIdx = d->model->indexForItem(parent);
-                if (parentIdx.isValid()) {
-                    emit d->model->dataChanged(parentIdx, parentIdx);
-                    parent = parent->parent();
-                } else {
-                    parent = nullptr;
-                }
+
+            auto *moduleData = KCModuleLoader::loadModuleData(item->item());
+            if (moduleData) {
+                connect(moduleData, &KCModuleData::loaded, this, [this, item, moduleData]() {
+                    item->setDefaultIndicator(!moduleData->isDefaults());
+                    updateModelMenuItem(item);
+                    moduleData->deleteLater();
+                });
             }
         }
     }
     config().writeEntry("HighlightNonDefaultSettings", d->m_defaultsIndicatorsVisible);
+    Q_EMIT defaultsIndicatorsVisibleChanged();
+}
 
-    emit defaultsIndicatorsVisibleChanged();
+void SidebarMode::updateModelMenuItem(MenuItem *item)
+{
+    auto itemIdx = d->model->indexForItem(item);
+    Q_EMIT d->model->dataChanged(itemIdx, itemIdx);
+    MenuItem *parent = item->parent();
+    while (parent) {
+        auto parentIdx = d->model->indexForItem(parent);
+        if (parentIdx.isValid()) {
+            Q_EMIT d->model->dataChanged(parentIdx, parentIdx);
+            parent = parent->parent();
+        } else {
+            parent = nullptr;
+        }
+    }
 }
 
 int SidebarMode::width() const
@@ -820,24 +838,27 @@ void SidebarMode::initWidget()
     d->moduleView->hide();
     d->mainLayout->addWidget(d->moduleView);
     d->mainLayout->addWidget(d->placeHolderWidget);
-    emit changeToolBarItems(BaseMode::NoItems);
+    Q_EMIT changeToolBarItems(BaseMode::NoItems);
 
     d->toolTipManager = new ToolTipManager(d->categorizedModel, d->quickWidget, ToolTipManager::ToolTipPosition::Right);
     d->mostUsedToolTipManager = new ToolTipManager(d->mostUsedModel, d->placeHolderWidget, ToolTipManager::ToolTipPosition::BottomCenter);
 
     d->mostUsedModel->setResultModel(new ResultModel(AllResources | Agent(QStringLiteral("org.kde.systemsettings")) | HighScoredFirst | Limit(5), this));
 
-    if (homeItem()) {
-        d->placeHolderWidget->hide();
-        d->moduleView->show();
-        loadModule(d->categorizedModel->mapFromSource(d->model->indexForItem(homeItem())));
-    }
-
     if (!startupModule().isEmpty()) {
         MenuItem *item = rootItem()->descendantForModule(startupModule());
         if (item) {
             loadModule(d->model->indexForItem(item), startupModuleArgs());
         }
+    } else if (homeItem()) {
+        d->placeHolderWidget->hide();
+        d->moduleView->show();
+        QModelIndex index = d->categorizedModel->mapFromSource(d->model->indexForItem(homeItem()));
+        if (index.isValid()) {
+            loadModule(index);
+        } else {
+            d->moduleView->loadModule( d->model->indexForItem(homeItem()), QStringList() );
+        }
     }
 }
 
@@ -875,9 +896,9 @@ bool SidebarMode::eventFilter(QObject *watched, QEvent *event)
     } else if (watched == d->quickWidget && event->type() == QEvent::Leave) {
         QCoreApplication::sendEvent(d->quickWidget->quickWindow(), event);
     } else if (watched == d->mainWidget && event->type() == QEvent::Resize) {
-        emit widthChanged();
+        Q_EMIT widthChanged();
     } else if (watched == d->mainWidget && event->type() == QEvent::Show) {
-        emit changeToolBarItems(BaseMode::NoItems);
+        Q_EMIT changeToolBarItems(BaseMode::NoItems);
     }
     return BaseMode::eventFilter(watched, event);
 }
